import lotterylib
import es

class ImproperlyConfigured(Exception):
    """
    Exception raised if you use a obviously bad API key or server key.
    """

"""
Load the api-key and server-key for this server/user. I don't do this in load
because I prefer the myapp being in the global scope without having to use the
'global' keyword.
"""
def load_config():
    """
    The layout of our config file is dead simple:
    If the line starts with 'api key:' (case insensitive) the stuff after the
    colon (whitespaces stripped) is considered the api key.
    The same goes for 'server key:' for the server key respectively.
    The default file uses JavaScript style comments so as not to confuse the 
    end user.
    We also do a very simple regex-check of the key if it's possibly correct.
    """
    import path
    import re
    key_pattern = re.compile(r'[a-zA-Z0-9]{64}')
    cfgfile = path.path(__file__).dirname().joinpath('keys.txt')
    lines = cfgfile.lines(retain=False)
    api_key = ''
    server_key = ''
    for line in lines:
        if ':' in line:
            start, end = line.split(':', 1)
            if start.lower().strip() == 'api key':
                api_key = end.strip()
            elif start.lower().strip() == 'server key':
                server_key = end.strip()
    if not key_pattern.match(api_key):
        raise ImproperlyConfigured, "Invalid API Key: '%s'" % api_key
    if not key_pattern.match(server_key):
        raise ImproperlyConfigured, "Invalid Server Key: '%s'" % server_key
    return api_key, server_key
api_key, server_key = load_config()
"""
The application key from esl.ojii.ch. This is constant.
"""
APP_KEY = 'jlF0r2rE1MTTiNNeG9z2LuotIovUqMqzqAGLSocWWbItkETTb2ivvVEifi6QGNNu'

"""
Get an instance of lotterylib.Application for our application, user and server.
"""
myapp = lotterylib.Application(api_key, server_key, APP_KEY)

class AutoDict(dict):
    """
    Yes this is hacky, but do I care?
    """
    def __getitem__(self, item):
        if not dict.__contains__(self, item):
            dict.__setitem__(self, 0)
        return dict.__getitem__(self, item)

class Cache(object):
    """
    The core of the addon.
    Keeps a dictionary (AutoDict) of players with steamids as keys and digits as
    values.
    It also handles responses from esl.ojii.ch and spawning players
    """
    def __init__(self):
        self.players = AutoDict()

    def handle_response(self, response):
        """
        Handle a response from es.ojii.ch.
        If the response is unsuccessful, we just raise the error. Quick'n'dirty.
        If it's successful, we just add '1' to each player in the winners list.
        """
        if not response.success:
            response.raise_error()
        else:
            winners = response.result.get('16k', [])
            for winner in winners:
                self.players[winner] += 1

    def handle_spawn(self, steamid, userid):
        """
        When a player spawns, we check if the player is in the dictionary.
        If they are, we give him 16k cash and reduce his value in the dictionary
        by 1. If the new value for that user is 0, we remove them.
        """
        if steamid in self.players:
            es.setplayerprop(userid, "CCSPlayer.m_iAccount", 16000)
            self.players[steamid] -= 1
            if not self.players[steamid]:
                del self.players[steamid]
cache = Cache()


def es_map_start(ev):
    """
    Whenever a new map starts, we get the new winning queue
    """
    myapp.get_queue(cache.handle_response)

def player_spawn(ev):
    """
    Whenever a player spawns, we check if we give them money.
    """
    cache.handle_spawn(ev['es_steamid'], ev['userid'])
